#!/usr/bin/env python
# −*− coding: UTF−8 −*−
import sys

def get_zone(c, name=None):
    zlist = []
    args = {}
    if name:
        args['name'] = name
    for z in c.list_zones(**args):
            zlist.append(z)
    if zlist:
        return zlist
    else:
        print "The search returned no zones"
        sys.exit(0)





#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage templates in RedBridge Cloud

Usage:
  rbc-templates list [-n] [--short] [--search=SEARCH] [FILTER]
  rbc-templates delete [-n] [--short] TEMPLATE

Arguments:
  FILTER   Template filter, valid options are:
           "featured", "self", "selfexecutable","sharedexecutable","executable", and "community".
           See the RedBridge Cloud API documentation for info.
           [default: executable]
  TEMPLATE Template name

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  --search=SEARCH                    Search templates matching STRING 
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator
from tabulate import tabulate
from config import read_config
from connection import conn
from docopt import docopt

__cli_name__="rbc-templates"
from rbc_tools import __version__

def list_templates(c, args):
    list_args = {} 
    if args['--search']:
        list_args['keyword'] = args['--search']
    list_args['templatefilter'] = args['FILTER']
    list_args['listall'] = "true"
    return c.list_templates(**list_args)

def delete_template(c, args):
    list_args = {} 
    list_args['id'] = get_template(c, args['TEMPLATE'])[0].id
    job = c.delete_template(**list_args)
    if job.get_result() == 'succeded':
        print "Template %s deleted" % args['TEMPLATE']
        sys.exit(0)
    else:
        print "Error deleting template"
        sys.exit(1)

def get_template_from_id(c ,id):
    try:
        return c.list_templates(templatefilter='executable', id=id)[0]
    except:
        try:
            return c.list_isos(id=id)[0]
        except:
            return False

def get_template(c, t):
    tlist = []
    try:
        account, name = t.split('/', 1)
    except ValueError:
        print "String must be in the format account/name"
        sys.exit(0)
    for t in c.list_templates(name=name, templatefilter='executable', listall="true"):
        tlist.append(t)
    if tlist:
        return tlist
    else:
        print "The search returned no templates"
        sys.exit(0)

def print_tabulate(res, noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if not i.__dict__.has_key('passwordenabled'):
            i.__setattr__('passwordenabled', 0)
        if i.passwordenabled == 1:
            passw = "Yes"
        else:
            passw = "No"
        if short:
            tbl.append(["%s/%s" % (i.account, i.name)])
        else:
            tbl.append([
                "%s/%s" % (i.account, i.name),
                i.zonename,
                i.ostypename,
                i.created,
                passw,
                ])

    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['name', 'zone', 'ostype', 'created', 'passwordenabled'])
        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if not args['FILTER']:
        args['FILTER'] = 'executable' 
    if not args['FILTER'] in ["featured", "self", "selfexecutable","sharedexecutable","executable", "community"]:
        print "Not a valid template filter"
        sys.exit(1)
    
    if args['list']:
        res = list_templates(c, args)
    if args['delete']:
        res = delete_template(c, args)

    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage remote access VPN's in RedBridge Cloud

Usage:
  rbc-vpns list [-n] [--short]
  rbc-vpns enable NETWORK
  rbc-vpns disable NETWORK
  rbc-vpns users list [-n] [--short]
  rbc-vpns users add USERNAME [PASSWORD]
  rbc-vpns users remove USERNAME
  rbc-vpns users password USERNAME PASSWORD

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator, os, random, string
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-vpns"
from rbc_tools import __version__

def get_network_for_ip(c, ipid):
    ip =  c.list_publicipaddresses(id=ipid)[0]
    if not ip.__dict__.has_key('associatednetworkname'):
        if ip.__dict__.has_key('vpcid'):
            ip.__setattr__('associatednetworkname', "%s (VPC)" % c.list_vpcs(id=ip.vpcid)[0].name)
    return ip.associatednetworkname

def get_network_from_name(c, name):
    ret = c.list_networks(keyword=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get network, got: %s" % ret
        sys.exit(1)

def get_publicip_for_network(c, name):
    net = get_network_from_name(c, name)
    ip = c.list_publicipaddresses(associatednetworkid=net.id, issourcenat='true')
    if len(ip) == 1:
        return ip[0]
    else:
        print "Unable to get ip address"
        sys.exit(1)

def print_tabulate(res, noheader=False, short=False, t="vpn"):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if t == 'vpn':
            if short:
                tbl.append([i.publicip])
            else:
                tbl.append([
                    get_network_for_ip(c, i.publicipid),
                    i.publicip,
                    i.presharedkey,
                    i.state,
                    ])
        if t == 'vpnusers':
            if short:
                tbl.append([i.username])
            else:
                tbl.append([
                    i.username,
                    i.state,
                    ])
    
    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        if t == 'vpn':
            tbl.insert(0, ['network', 'endpoint', 'preshared key', 'state'])
        elif t == 'vpnuser':
            tbl.insert(0, ['username', 'state'])
        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if (args['list'] and not args['users']):
        res = c.list_remoteaccessvpns()
        t = "vpn"
    elif args['enable']:
        ip = get_publicip_for_network(c, args['NETWORK'])
        try:
            ret = c.create_remoteaccessvpn(publicipid=ip.id)
        except Exception as e:
            print "Unable to enable remote access VPN: %s" % e[1]
            sys.exit(1)
    elif args['disable']:
        ip = get_publicip_for_network(c, args['NETWORK'])
        try:
            ret = c.delete_remoteaccessvpn(publicipid=ip.id)
        except Exception as e:
            print "Unable to disable remote access VPN: %s" % e[1]
            sys.exit(1)
    elif args['users']:
        t = 'vpnusers'
        if args['list']:
            res = c.list_vpnusers()
        if args['add']:
            if not args['PASSWORD']:
                length = 8
                chars = string.ascii_letters + string.digits + '@#+=.-_'
                random.seed = (os.urandom(1024))
                passwd = ''.join(random.choice(chars) for i in range(length))
            else:
                passwd = args['PASSWORD']
            c.add_vpnuser(username=args['USERNAME'], password=passwd)
            print "VPN user %s added with password %s" % (args['USERNAME'], passwd)
            sys.exit(1) 
        elif args['remove']:
            try:
                c.remove_vpnuser(username=args['USERNAME'])
                print "Removed user: %s" % args['USERNAME']
                sys.exit(0)
            except Exception as e:
                print "Unable to remove VPN user: %s" % e[1]
                sys.exit(1)
    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'], t=t)
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage networks in RedBridge Cloud

Usage:
  rbc-networks list [-n] [--short] [--search=SEARCH]
  rbc-networks create [-n] [--short] [-d DOMAIN] [-z ZONE] -o OFFERING NETWORK
  rbc-networks delete [-n] [--short] NETWORK

Arguments:
  NETWORK network name

Options:
  -n --noheader                      Do not show the table header
  --short                            Short disply, only shows name
  --search=SEARCH                    Search networks matching SEARCH
  -o OFFERING --offering=OFFERING    Name of network offering
  -d DOMAIN --domain=DOMAIN          Name of internal domain
  -z ZONE --zone=ZONE                Name of zone
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-networks"
from rbc_tools import __version__

def get_offering_id(c, name):
    try:
        for i in c.list_networkofferings(name=name):
            if i.name == name:
                return i.id
    except:
        print "Unable to get id for offering"
        sys.exit(1)

def get_snat(c, id):
    ip = c.list_publicipaddresses(associatednetworkid=id, issourcenat="true")
    if ip:
        return ','.join([x.ipaddress for x in ip])
    else:
        return ['N/A']

def has_vpn(c, id):
    v = c.list_remoteaccessvpns(networkid=id)
    if isinstance(v, dict):
        if v.has_key('count'):
            return "Yes"
    else:
        return "No"

def get_network(c, name):
    nlist = []
    for n in c.list_networks(canusefordeploy="true"):
        if n.name == name:
            nlist.append(n)
    if nlist:
        return nlist
    else:
        print "The search returned no networks"
        sys.exit(0)

def print_tabulate(res, noheader=False, short=False):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if short:
            tbl.append([i.name])
        else:
            tbl.append([
                i.name,
                i.zonename,
                i.networkofferingname,
                i.cidr,
                i.networkdomain,
                get_snat(c, i.id),
                has_vpn(c, i.id),
                i.state
                ])

    tbl = sorted(tbl, key=operator.itemgetter(0))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['name', 'zone', 'offering', 'cidr', 'domain', 'snat', 'vpn', 'state'])
        print tabulate(tbl, headers="firstrow")


def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        list_args ={'listall': 'true'}
        if args['--search']:
            list_args['keyword'] = args['--search']
        res = c.list_networks(**list_args)
    if args['delete']:
        id = get_network(c, args['NETWORK'])[0].id
        if id:
            res = c.delete_network(id=id).get_result()
            if res:
                print "network deleted"
                sys.exit(0)
            else:
                print "failed to delete the network, check that there are no running instances deployed on the network."
                sys.exit(1)
    if args['create']:
        create_args = {'name': args['NETWORK'], 'displaytext': args['NETWORK']}
        if args['--offering']:
            create_args['networkofferingid'] = get_offering_id(c, args['--offering'])
        if args['--domain']:
            create_args['networkdomain'] = args['--domain']
        if args['--zone']:
            zone = get_zone(c, args['--zone'])[0]
            create_args['zoneid'] = zone.id
        else:
            zone = get_zone(c)[0]
            create_args['zoneid'] = zone.id
            res = c.create_network(**create_args)
            if res:
                print "network created"
                sys.exit(0)

    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)



#!/usr/bin/env python
# −*− coding: UTF−8 −*−
"""Manage static NAT mappings in RedBridge Cloud

Usage:
  rbc-staticnats list [-n] [--short] [NETWORK]
  rbc-staticnats enable INSTANCE IPADDRESS [--rules RULE...]
  rbc-staticnats disable IPADDRESS

Options:
  -n --noheader  Do not show the table header
  --short        Short disply, only shows name
  --open=RULE    A fw rule in the format src:proto/port (0.0.0.0/0:tcp/22)
  -h --help      Show this screen.
  --version      Show version.

"""
import sys, operator, os, random, string
from tabulate import tabulate
from config import read_config
from connection import conn
from zone import get_zone
from docopt import docopt

__cli_name__="rbc-staticnats"
from rbc_tools import __version__

def get_network_from_name(c, name):
    ret = c.list_networks(keyword=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get network, got: %s" % ret
        sys.exit(1)

def get_instance_from_name(c, name):
    ret = c.list_virtualmachines(keyword=name, name=name)
    if len(ret) == 1:
        return ret[0]
    else:
        print "Unable to get instance, got: %s" % ret
        sys.exit(1)

def get_id_from_address(c, address):
    try:
        res = c.list_publicipaddresses(ipaddress=address)[0]
        if res:
            return res
        else:
            print "Unable to get ip address id"
            sys.exit(0)
    except:
        print "Unable to get ip address id"
        sys.exit(1)

def get_firewall_rules(c, ipid):
    return c.list_firewallrules(ipaddressid=ipid)

def rule_list_to_string(rules):
    r = []
    for i in rules:
        r.append("%s:%s/%s" % (i.cidrlist, i.protocol, i.startport))
    return " ".join(r)

def print_tabulate(res, noheader=False, short=False, t='list'):
    config = read_config()
    c = conn(config)
    tbl = []
    for i in res:
        if t == 'list':
            rules = get_firewall_rules(c, ipid=i.id)
            r = rule_list_to_string(rules)
            if not i.__dict__.has_key('associatednetworkname'):
                if i.__dict__.has_key('vpcid'):
                    i.__setattr__('associatednetworkname', "%s (VPC)" % c.list_vpcs(id=i.vpcid)[0].name)
            if i.isstaticnat:
                vmname = i.virtualmachinename
            else:
                vmname = ""
            if i.issourcenat:
                snat = "Yes"
            else:
                snat = "No"
            if short:
                tbl.append([i.name])
            else:
                tbl.append([
                    i.ipaddress,
                    i.zonename,
                    i.associatednetworkname,
                    vmname,
                    i.allocated,
                    r
                ])

    tbl = sorted(tbl, key=operator.itemgetter(2))
    if (noheader or short):
        print tabulate(tbl, tablefmt="plain")
    else:
        tbl.insert(0, ['ip', 'zone', 'network', 'staticnat', 'allocated', 'rules'])
        print tabulate(tbl, headers="firstrow")

def main():
    config = read_config()
    c = conn(config)
    args = docopt(__doc__, version='%s %s' % (__cli_name__, __version__))
    if args['list']:
        if args['NETWORK']:
            res = c.list_publicipaddresses(networkid=get_network_from_name(c, args['NETWORK']).id, isstaticnat='true')
        else:
            res = c.list_publicipaddresses(isstaticnat='true')
    elif args['enable']:
        try:
            if c.enable_staticnat(ipaddressid=get_id_from_address(c, args['IPADDRESS']).id, virtualmachineid=get_instance_from_name(c, args['INSTANCE']).id):
                if args['--rules']:
                    print args
                    try:
                        for rule in args['RULE']:
                            src = rule.split(':')
                            proto_port = src[1].split('/')
                            c.create_firewallrule(ipaddressid=get_id_from_address(c, args['IPADDRESS']).id, startport=proto_port[1], protocol=proto_port[0], cidrlist=src[0])
                    except Exception as e:
                        print "Unable create firewall rules: %s" % e
                print "Enabled static nat for %s on %s" % (args['INSTANCE'], args['IPADDRESS'])
                sys.exit(0)
        except Exception as e:
            print "Unable to enable static nat: %s" % e[1]
            sys.exit(1)
    elif args['disable']:
        try:
            res = c.disable_staticnat(ipaddressid=get_id_from_address(c, args['IPADDRESS']).id).get_result()
            if res == 'succeded':
                print "Disabled static nat on %s" % args['IPADDRESS']
                sys.exit(0)
        except Exception as e:
            print "Unable to disable static nat: %s" % e[1]
            sys.exit(1)

    if res:
        print_tabulate(res, noheader=args['--noheader'], short=args['--short'])
    else:
        print "Unable to execute command"
        sys.exit(1)
